/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Vicente Sanjaime (vsanjaime@prodevelop.es)
 */

package es.prodevelop.gvsig.mini.geom.j2se;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.geom.LinearRing;

import es.prodevelop.geodetic.utils.conversion.ConversionCoords;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mobile.fmap.proj.CRSFactory;
import es.prodevelop.gvsig.mobile.fmap.proj.Projection;

/**
 * An extent that implements a new method to check if it intersects with a
 * JTSPolygon
 * 
 * @author vsanjaime
 * 
 */
public class AdvancedExtent extends Extent {

	/**
	 * Constructor
	 * 
	 * @param minX
	 * @param minY
	 * @param maxX
	 * @param maxY
	 */
	public AdvancedExtent(double minX, double minY, double maxX, double maxY) {
		super(minX, minY, maxX, maxY);
	}

	/**
	 * Validate the intersection with another complex polygon
	 * 
	 * @param poly
	 *            (JTS object)
	 * @return
	 */
	public boolean intersectWithPolygon(Geometry poly) {
		Coordinate[] coords = new Coordinate[5];
		coords[0] = new Coordinate(super.getMinX(), super.getMinY());
		coords[1] = new Coordinate(super.getMinX(), super.getMinY()
				+ super.getHeight());
		coords[2] = new Coordinate(super.getMinX() + super.getWidth(), super
				.getMinY()
				+ super.getHeight());
		coords[3] = new Coordinate(super.getMinX() + super.getWidth(), super
				.getMinY());
		coords[4] = new Coordinate(super.getMinX(), super.getMinY());
		GeometryFactory factory = new GeometryFactory();
		LinearRing shell = factory.createLinearRing(coords);
		LinearRing[] holes = new LinearRing[0];
		Geometry bbox = factory.createPolygon(shell, holes);
		bbox.intersects(poly);

		return bbox.intersects(poly);

	}

	/**
	 * Validate the intersection with another complex polygon. This method
	 * automatically converts the coordinates of the AdvancedExtent to the
	 * Geometry SRS
	 * 
	 * @param poly
	 *            (JTS object)
	 * @param fromSRS
	 *            The EPSG code of the SRS of the current AdvancedExtent *
	 * @param toSRS
	 *            The EPSG code of the SRS of the coordinates of the Geometry
	 * @return
	 */
	public boolean intersectWithPolygon(Geometry poly, String fromSRS,
			String toSRS) {
		
		Projection fromProj = CRSFactory.getCRS(fromSRS);
		Projection toProj = CRSFactory.getCRS(toSRS);
		
		double[] minXY = ConversionCoords.reproject(getMinX(), getMinY(), fromProj, toProj);
		double[] maxXY = ConversionCoords.reproject(getMaxX(), getMaxY(), fromProj, toProj);
		
		Extent reprojectedExtent = new Extent(minXY[0], minXY[1], maxXY[0], maxXY[1]);
		
		Coordinate[] coords = new Coordinate[5];
		coords[0] = new Coordinate(reprojectedExtent.getMinX(), reprojectedExtent.getMinY());
		coords[1] = new Coordinate(reprojectedExtent.getMinX(), reprojectedExtent.getMinY()
				+ reprojectedExtent.getHeight());
		coords[2] = new Coordinate(reprojectedExtent.getMinX() + reprojectedExtent.getWidth(), reprojectedExtent
				.getMinY()
				+ reprojectedExtent.getHeight());
		coords[3] = new Coordinate(reprojectedExtent.getMinX() + reprojectedExtent.getWidth(), reprojectedExtent
				.getMinY());
		coords[4] = new Coordinate(reprojectedExtent.getMinX(), reprojectedExtent.getMinY());
		GeometryFactory factory = new GeometryFactory();
		LinearRing shell = factory.createLinearRing(coords);
		LinearRing[] holes = new LinearRing[0];
		Geometry bbox = factory.createPolygon(shell, holes);		

		return bbox.intersects(poly);

	}

}
