/* gvSIG Mini. A free mobile phone viewer of free maps.
 *
 * Copyright (C) 2010 Prodevelop.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *   Prodevelop, S.L.
 *   Pza. Don Juan de Villarrasa, 14 - 5
 *   46001 Valencia
 *   Spain
 *
 *   +34 963 510 612
 *   +34 963 510 968
 *   prode@prodevelop.es
 *   http://www.prodevelop.es
 *
 *   gvSIG Mini has been partially funded by IMPIVA (Instituto de la Peque�a y
 *   Mediana Empresa de la Comunidad Valenciana) &
 *   European Union FEDER funds.
 *   
 *   2010.
 *   author Alberto Romeu aromeu@prodevelop.es
 */

package es.prodevelop.tilecache.j2se;

import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Polygon;

import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.j2se.AdvancedExtent;
import es.prodevelop.tilecache.generator.ITileBufferIntersector;

/**
 * A simple implementation of ITileBufferIntersector that checks if an Extent
 * intersects with a JTS polygon
 * 
 * @author aromeu
 * 
 */
public class JTSPolygonIntersector implements ITileBufferIntersector {

	private Geometry polygon;
	private String fromSRS;
	private String toSRS;

	/**
	 * The constructor
	 * 
	 * @param polygonToIntersectWith
	 *            A JTS polygon
	 * @param fromSRS
	 *            The EPSG code of the SRS of the current AdvancedExtent
	 * @param toSRS
	 *            The EPSG code of the SRS of the coordinates of the Geometry
	 */
	public JTSPolygonIntersector(Geometry polygonToIntersectWith,
			String fromSRS, String toSRS) {
		this.polygon = polygonToIntersectWith;
		this.fromSRS = fromSRS;
		this.toSRS = toSRS;
	}

	public boolean intersects(Extent extent) {
		AdvancedExtent e = new AdvancedExtent(extent.getMinX(), extent
				.getMinY(), extent.getMaxX(), extent.getMaxY());
		return e.intersectWithPolygon(this.polygon, this.fromSRS, this.toSRS);
	}

}
