package es.prodevelop.tilecache.j2se;

import es.prodevelop.gvsig.mini.common.CompatManager;
import es.prodevelop.gvsig.mini.common.j2se.impl.J2SEContext;
import es.prodevelop.gvsig.mini.common.j2se.impl.VoidLogHandler;
import es.prodevelop.gvsig.mini.exceptions.BaseException;
import es.prodevelop.gvsig.mini.geom.Extent;
import es.prodevelop.gvsig.mini.geom.Point;
import es.prodevelop.gvsig.mini.projection.TileConversor;
import es.prodevelop.gvsig.mini.utiles.Cancellable;
import es.prodevelop.gvsig.mini.utiles.WorkQueue;
import es.prodevelop.tilecache.TileDownloaderTask;
import es.prodevelop.tilecache.generator.impl.FitScreenBufferStrategy;
import es.prodevelop.tilecache.layers.Layers;
import es.prodevelop.tilecache.provider.TileProvider;
import es.prodevelop.tilecache.provider.filesystem.strategy.ITileFileSystemStrategy;
import es.prodevelop.tilecache.provider.filesystem.strategy.impl.FlatXFileSystemStrategy;
import es.prodevelop.tilecache.renderer.MapRenderer;
import es.prodevelop.tilecache.renderer.MapRendererFactory;
import es.prodevelop.tilecache.renderer.MapRendererManager;
import es.prodevelop.tilecache.summary.SummaryInfo;
import es.prodevelop.tilecache.util.ConstantsTileCache;
import es.prodevelop.tilecache.util.Utilities;

public class Main {

	static TileDownloaderTask tileDownloaderTask;

	public static void main(String[] params) {
		try {
			
			int minZoomLevel = -1;
			int maxZoomLevel = -1;
			Point p1 = null;
			Point p2 = null;
			String layerName = "OPEN STREET MAP";
			
			if (params.length == 0) {
				System.err.println("Wrong parameters...");
				System.exit(-1);
			} else {
				minZoomLevel = Integer.parseInt(params[0]);
				maxZoomLevel = Integer.parseInt(params[1]);
				double minX = Double.parseDouble(params[2]);
				double minY = Double.parseDouble(params[3]);
				double maxX = Double.parseDouble(params[4]);
				double maxY = Double.parseDouble(params[5]);
				
				p1 = TileConversor.latLonToMercator(minX,
						minY);
				p2 = TileConversor.latLonToMercator(maxX,
						maxY);
				layerName = params[6];
			}
				

			/*
			 * Initialize managers and feed the Layers instance with the
			 * layers.txt file
			 */
			MapRendererManager.getInstance().registerMapRendererFactory(
					new MapRendererFactory());
			CompatManager.getInstance().registerContext(new J2SEContext());
			CompatManager.getInstance().registerLogHandler(new VoidLogHandler());
			Layers.getInstance().initialize(false);

			/*
			 * Instantiate the default OSM renderer (to test). Any other can be
			 * taken from Layers.getInstance().getRenderer(layerTitle) Vector
			 * layers = (Vector) Layers.getInstance().getLayersForView().get(0);
			 * renderer =
			 * Layers.getInstance().getRenderer(layers.get(16).toString());
			 */
			MapRenderer renderer = Layers.getInstance().getRenderer(layerName);

			// Vector layers = (Vector) Layers.getInstance().getLayersForView()
			// .get(1);
			// renderer = Layers.getInstance().getRenderer(
			// layers.get(0).toString());

			/*
			 * We have to set the extent to the MapRenderer. The coordinates
			 * must be in the same SRS as the MapRenderer @see
			 * MapRenderer.getSRS();
			 */

			// Valencia
			// Point p1 = TileConversor.latLonToMercator(-0.43, 39.4);
			// Point p2 = TileConversor.latLonToMercator(-0.3, 39.53);
			// Paris
//			Point p1 = TileConversor.latLonToMercator(2.109375,
//					48.69096039092549);
//			Point p2 = TileConversor.latLonToMercator(2.4609375,
//					48.922499263758255);
			

			// Spain
			// Point p1 = TileConversor.latLonToMercator(-19.00000000000001,
			// 27.0000000442298);
			// Point p2 = TileConversor.latLonToMercator(4.999999999999999,
			// 44.00000008359081);
			Extent rendererExtent = renderer.getExtent();

			renderer.setExtent(new Extent(p1, p2));

			/*
			 * Once we set the extent, we have to update the center of the
			 * MapRenderer
			 */
			renderer.setCenter(renderer.getExtent().getCenter().getX(),
					renderer.getExtent().getCenter().getY());

			/*
			 * Get a new Cancellable instance (one different each time the task
			 * is executed)
			 */
			Cancellable c = Utilities.getNewCancellable();

//			int minZoomLevel = renderer.getZoomMinLevel();
//			int maxZoomLevel = 15;
			Extent e = renderer.getExtent();

			ITileFileSystemStrategy strat = new FlatXFileSystemStrategy();

			SummaryInfo summaryInfo = new SummaryInfo(renderer, minZoomLevel,
					maxZoomLevel, e, "1", strat.getLayerNameSuffix(), "test_city", rendererExtent);

			/*
			 * Instantiate the download waiter. The implementation should
			 * properly update the UI as the task is processing tiles
			 */
			DefaultDownloadWaiter downloadWaiter = new DefaultDownloadWaiter(
					true, summaryInfo);
			DefaultDownloadCallbackHandler callBackHandler = new DefaultDownloadCallbackHandler(
					downloadWaiter);
			ConstantsTileCache.DEFAULT_NUM_THREADS = 50;

			/*
			 * Finally instantiate the TileDownloaderTask. Automatically
			 * launches to the IDownloadWaiter implementation the
			 * onTotalNumTilesRetrieved event (prior to execute the task)
			 * 
			 * This instance will donwload from 0 to 3 zoom level of the whole
			 * world extent of OSM (Mapnik) server. Not apply any ITileSorter
			 * nor ITileBufferIntersector and applies the FlatX file system
			 * strategy with a minimum buffer to fit the map size.
			 */
			TileDownloaderTask t = new TileDownloaderTask(CompatManager
					.getInstance().getRegisteredContext(), renderer,
					minZoomLevel, maxZoomLevel, c, e, null, callBackHandler,
					null, TileProvider.MODE_ONLINE, strat,
					new FitScreenBufferStrategy(), true);

			/*
			 * Launch the process on a thread, then take care of the
			 * IDownloadWaiter events to update the UI
			 */
			WorkQueue.getExclusiveInstance().execute(t);
		} catch (BaseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
